#include "eram.h"
#include "pdm.h"
#include "audio.h"


typedef enum
{
    ERAM_CMD_READ = 0x03,
    ERAM_CMD_WRITE = 0x02,
    ERAM_CMD_RDMR = 0x05,
    ERAM_CMD_WRMR = 0x01,
} ERAM_CMD;


//#define ERAM_UNIT_TEST
void initERAM(void)
{
    initSPI1();
   
#ifdef ERAM_UNIT_TEST   
    printk("-> ERAM: begin unit test\n");
    uint8_t data[10];
    uint16_t k = 0;
    for(k = 0; k < 10; k++)
    {
        data[k] = k;
    }
    writeDataERAM(0x000000,data,10);
    //delay_ms(1000);
    for(k = 0; k < 10; k++)
    {
        data[k] = 0xff;
    }
    readDataERAM(0x000000,data,10);
    for(k = 0; k < 10; k++)
    {
        printk("[%d]%d\n",k,data[k]);
    }
    printk("\n");
    while(1);
#endif    

}

void writeModeRegERAM(uint8_t data)
{
    SPI1_CS_LOW;
    txByteSSPI(ERAM_CMD_WRMR);
    txByteSSPI(data);
    SPI1_CS_HIGH;
}

uint8_t readModeRegERAM()
{
    /* ERAM_CMD_RDMR = 0x05, */
    /* ERAM_CMD_WRMR = 0x01, */
    SPI1_CS_LOW;
    txByteSSPI(ERAM_CMD_RDMR);
    uint8_t data;
    data = rxByteSSPI();
    SPI1_CS_HIGH;
    
    return data;
}

void writeDataERAM(uint32_t addr, uint8_t * data, uint16_t size)
{
    SPI1_CS_LOW;
    txByteSSPI(ERAM_CMD_WRITE);
    txByteSSPI((addr >> 16) & 0xff);
    txByteSSPI((addr >> 8) & 0xff);
    txByteSSPI((addr) & 0xff);
    txDataSPI1(data,size);
    SPI1_CS_HIGH;
}

void readDataERAM(uint32_t addr, uint8_t * data, uint16_t size)
{
    SPI1_CS_LOW;
    txByteSSPI(ERAM_CMD_READ);
    txByteSSPI((addr >> 16) & 0xff);
    txByteSSPI((addr >> 8) & 0xff);
    txByteSSPI((addr) & 0xff);
    rxDataSPI1(data,size);
    SPI1_CS_HIGH;
}

void chkERAM(void)
{
	uint8_t * data;
	uint32_t addr = g_AUDIO_INFO.widx*2;
	uint32_t size = 2*(PDM_DMA_BUFF_SIZE);
	if(g_PDM_INFO.bflag)
	{
		data = (uint8_t *)g_AUDIO_INFO.buff1;
	}
	else
	{
		data = (uint8_t *)g_AUDIO_INFO.buff0;
	}

	// check the address if it will go beyond die addr boundary
	if((addr < (ERAM_DIE_ADDR_BOUNDARY))&&((addr+size) > (ERAM_DIE_ADDR_BOUNDARY)))
	{
		uint32_t tsize = (ERAM_DIE_ADDR_BOUNDARY) - addr;
		// write segment in first die
		writeDataERAM(addr, data, tsize);
		
		// write remaining in second die
		writeDataERAM(ERAM_DIE_ADDR_BOUNDARY, &data[tsize], size - tsize);
		
		printk("-> ERAM: write boundary case:\n\tADD: %08X\n\t1WR: %d\n\t2WR: %d\n",addr,tsize, size-tsize);
	}
	else
	{
		//printk("-> ERAM: A:%08X B:%08X S:%d\n",addr, (uint32_t)data, size);	
		writeDataERAM(addr, data, size);
	}
	
	g_AUDIO_INFO.widx += (PDM_DMA_BUFF_SIZE);
	
}
