#include <zephyr/types.h>
#include <stddef.h>
#include <string.h>
#include <errno.h>
#include <sys/printk.h>
#include <sys/byteorder.h>
#include <zephyr.h>
#include <bluetooth/l2cap.h>
#include "delay.h"
#include "blef.h"
#include "bleas.h"
#include "cmd.h"
#include "audio.h"
#include "eram.h"
#include "sensor.h"
#include "ADXL345.h"

//#include <ztest.h>

//#include <zephyr/l2cap.h>
static int  init(void);
static void app_main(void);

// aplay audio.bin -r 16000 -f S16_LE -t raw -c 1
#define APP_TREAD_STACKSIZE     2048
#define APP_TREAD_PRIORITY      2

K_THREAD_DEFINE(thread_app, APP_TREAD_STACKSIZE, app_main, NULL, NULL, NULL, APP_TREAD_PRIORITY, 0, 0);


//wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww
// CENTRAL CONNECTION SPECIFICATION:
//wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww
// MTU SIZE: 					215
// DLE:							enabled (data length extension enabled)
// BLE GAP EVENT LENGTH: 		2.5 ms (2x 1.25ms)
// SLAVE LATENCY:        		0 
// CONNECTION INTERVAL MIN:		7.5 ms
// CONNECTION INTERVAL MAX:		10 ms
// 1. make sure that connection parameters are updated from central side to best
//    case scenario
// 2. make sure that central does not have peripheral role acive which eats up the
//    BLE time and hence degrades the performance
// 3. make sure that the code is built with real time priory (when compiling the code)
//wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww
// DEVICE CONNECTION SPECIFICATION:
//wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww
//CONFIG_BT_L2CAP_TX_BUF_COUNT=16
//CONFIG_BT_PERIPHERAL_PREF_MIN_INT=6
//CONFIG_BT_PERIPHERAL_PREF_MAX_INT=9
static int l2cap_accept(struct bt_conn *conn, struct bt_l2cap_chan **chan)
{
	return -ENOSYS;
}

static struct bt_l2cap_server test_server = {
	.accept		= l2cap_accept,
};

static struct bt_l2cap_server test_fixed_server = {
	.accept		= l2cap_accept,
	.psm		= 0x007f,
};

static struct bt_l2cap_server test_dyn_server = {
	.accept		= l2cap_accept,
	.psm		= 0x0055,
};

static struct bt_l2cap_server test_inv_server = {
	.accept		= l2cap_accept,
	.psm		= 0xffff,
};
void test_l2cap_register(void)
{



	/* Attempt to register server with dynamic PSM */
	bt_l2cap_server_register(&test_dyn_server);



}

//wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww
static void app_main(void) 
{
	
	//I2C_routine();
	//k_msleep(500);
	//Enable_Meas();
	//setRange(ADXL345_RANGE_16_G);
	
	//while(1)
	//{
		//uint16_t x = 0, y = 0, z = 0;
		//x = getX();
		//y = getY();
		//z = getZ();
		//printk("-> SYS: %d %d %d\n",x,y,z);
		//k_msleep(500);
	//}
	
	unsigned int signaled;
    int          result;  
    
    if(initBLE())
	{
		while(1);
	}
	
	initCMD();
	initSYSTIM(1);
	initERAM();
	initAUDIO();
	// init ACCELEROMETER low lever driver which will collect the data
	//initSENSOR();
	
    k_poll_signal_init(&signal_audio_action);
    k_poll_signal_init(&signal_eram_action);
    k_poll_signal_init(&signal_sensor_action);
    k_poll_signal_init(&signal_accel_action);
    k_poll_signal_init(&signal_udp_action);
	
    struct k_poll_event app_events[] = {
		K_POLL_EVENT_INITIALIZER(K_POLL_TYPE_SIGNAL, K_POLL_MODE_NOTIFY_ONLY, &signal_audio_action),
		K_POLL_EVENT_INITIALIZER(K_POLL_TYPE_SIGNAL, K_POLL_MODE_NOTIFY_ONLY, &signal_eram_action),
		K_POLL_EVENT_INITIALIZER(K_POLL_TYPE_SIGNAL, K_POLL_MODE_NOTIFY_ONLY, &signal_sensor_action),
		K_POLL_EVENT_INITIALIZER(K_POLL_TYPE_SIGNAL, K_POLL_MODE_NOTIFY_ONLY, &signal_accel_action),
		K_POLL_EVENT_INITIALIZER(K_POLL_TYPE_SIGNAL, K_POLL_MODE_NOTIFY_ONLY, &signal_udp_action),
	};
     
   
			 
    printk("-> SYS: runnig...\n");
    uint32_t cnt = 0;
    
    while(1)
	{
		//printk("-> SYS: %d...\n",cnt++);
		k_poll(app_events, ARRAY_SIZE(app_events), K_FOREVER);
		
		k_poll_signal_check(&signal_audio_action, &signaled, &result);
		if(signaled)
		{
            k_poll_signal_reset(&signal_audio_action);
			chkCMD();
			chkAUDIO();
		}
		
		k_poll_signal_check(&signal_eram_action, &signaled, &result);
		if(signaled)
		{// time to copy data from PDM buffer to ERAM
            k_poll_signal_reset(&signal_eram_action);
			chkERAM();
		}
		
		k_poll_signal_check(&signal_sensor_action, &signaled, &result);
		if(signaled)
		{// 
			k_poll_signal_reset(&signal_sensor_action);
			// ADD your code to process data received from the sensor characteristic
			// which you presumably stored into some intermediate buffer and 
		}
		
		
		k_poll_signal_check(&signal_accel_action, &signaled, &result);
		if(signaled)
		{// 
			k_poll_signal_reset(&signal_accel_action);
			
			//printk("-> SENSOR: accel handler...\n");
			//uint16_t x = 0, y = 0, z = 0;
			//x = getX();
			//y = getY();
			//z = getZ();
			uint16_t data[3];
			getData(data);
			 
			// now we add the collected accelerometer data to the FIFO
			//procAccelDataSENSOR(x, y, z);
			procAccelDataSENSOR(data[0], data[1], data[2]);
		}
		
		k_poll_signal_check(&signal_udp_action, &signaled, &result);
		if(signaled)
		{
			k_poll_signal_reset(&signal_udp_action);
			
			txUdpDataCMDAPI();
		}
		
		//k_msleep(100);
	}
}

//wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww
//w nRF52832 Hub device v[1.1.2]
//wwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwwww
//-> SYS: DID [ADD709FED731EC98]
//B1 00000000
//B2 00000000, RAM START20008000
//B5 00000000
//B3 00000000
//B4 00000000
//-> BLEC: searching for devices...
//-> BLE: MAC [DB:82:A1:57:1B:EC]
//-> BLE: init done...
//-> SYS: init completed...
//-> HUB: waiting...
//-> BLEC: ADV.size 11 ADDR F0:6E:08:77:33:21
//02 01 06 07 03 09 18 0A 18 0F 18 
//-> BLEC: connecting to peripheral device []...
//-> BLEC: PERI[0] connected
	 //HND  [0001]
	 //MAC  [F0:6E:08:77:33:21]
	 //NAME []
 //CCCD [0011]
//-> BLEC: searching for devices...
//BLE_GAP_EVT_CONN_PARAM_UPDATE_REQUEST. type 1-> BLEC: scan stopped...
//-> BLEC: enabling notifications for 1 peripherals...
//-> BLEC: enabled for PERI[00] HND[0001] CCDHND[0011]
//-> BLEC: enabled for PERI[00] HND[0001] CCDHND[0011]
		//PERI[0] Pkts 0/0 - Rb 0 b
//-> HUB: acquisition INIT...
//-> HUB: acquisition DONE...
//-> HUB: init download
//-> HUB: downloading file size 256000 B
		//PERI[0] Pkts 2213/0 - Rb 265608 b
		//PERI[0] Pkts 2212/0 - Rb 265512 b
		//PERI[0] Pkts 2356/0 - Rb 282800 b
		//PERI[0] Pkts 2437/0 - Rb 292448 b
		//PERI[0] Pkts 2452/0 - Rb 294344 b
		//PERI[0] Pkts 2517/0 - Rb 302040 b
		//PERI[0] Pkts 2148/0 - Rb 257816 b
		//PERI[0] Pkts 2388/0 - Rb 286648 b
		//PERI[0] Pkts 2469/0 - Rb 296296 b
		//PERI[0] Pkts 2469/0 - Rb 296296 b
		//PERI[0] Pkts 2212/0 - Rb 265512 b
		//PERI[0] Pkts 2452/0 - Rb 294344 b
		//PERI[0] Pkts 2501/0 - Rb 299844 b
		//PERI[0] Pkts 2437/0 - Rb 292448 b
//-> HUB: download completed  0003E83C - 0003E800 in 14614 ms 



